@@ -22,6 +22,7 @@ use fields (
 	'route',   # for 'route' header, comes usually from 'record-route' info in response
 	'via',     # for 'via' header in created responses, comes from incoming request
 	'incoming', # flag if call is incoming, e.g. 'to' is myself
+	'local_tag', # local tag which gets assigned to either from or to depending on incoming
 
 	# ===== Internals
 	# \@array of hashrefs for infos about pending transactions
@@ -43,7 +44,7 @@ use Net::SIP::Debug;
 use Errno qw( EINVAL EPERM EFAULT );
 use Hash::Util 'lock_keys';
 use List::Util 'first';
-use Net::SIP::Util qw(sip_hdrval2parts invoke_callback);
+use Net::SIP::Util ':all';
 
 ############################################################################
 # Creates new context
@@ -66,10 +67,16 @@ sub new {
 	$self->{_transactions} = [];
 	$self->{_cseq_incoming} = 0;
 
-	# create tag for from header
-	my ($from,$fparam) = sip_hdrval2parts( from => $self->{from} );
-	unless ( $fparam->{tag} ) {
-		$self->{from}.=";tag=".md5_hex( time(), rand(2**32), $self->{from} );
+	# create tag on my side (to|from)
+	my $side = $self->{incoming} ? 'to':'from';
+	my ($data,$param) = sip_hdrval2parts( $side => $self->{$side} );
+	if ( my $tag = $param->{tag} ) {
+		# FIXME: what to do if local_tag was already set to different value?
+		$self->{local_tag} = $tag;
+	} else {
+		$self->{$side}.=";tag=".(
+			$self->{local_tag} = md5_hex( time(), rand(2**32), $self->{$side} )
+		);
 	}
 
 	DEBUG( 100,"CREATE context $self" );
@@ -324,6 +331,13 @@ sub handle_response {
 			my $ack = $tr->{request}->create_ack( $response );
 			invoke_callback($cb,@arg,0,$code,$response,$leg,$from,$ack);
 			$endpoint->new_request( $ack,$self,undef,undef,leg => $leg, dst_addr => $from );
+
+			# use to-tag from this request to update 'to'
+			# FIXME: this should probably be better done by the upper layer
+			# which decides, which call to accept (in case of call-forking with
+			# multiple 2xx responses)
+			$self->{to} = $response->get_header( 'to' );
+
 		} else {
 			# response to ACK, REGISTER...
 			# simply propagate to upper layer, only INVITE needs
@@ -436,6 +450,24 @@ sub handle_request {
 	};
 	my @arg = ($endpoint,$self);
 
+	# extract route information for future requests to the UAC (re-invites)
+	if ( my @route = $request->get_header( 'record-route' )) {
+		$self->{route} = \@route;
+	}
+
+	{
+		# check if to has already a (my) tag, if not add it to request,
+		# so that it gets added to responses
+		my $to = $request->get_header( 'to' );
+		my ($data,$param) = sip_hdrval2parts( to => $to );
+		if ( ! $param->{tag} ) {
+			DEBUG( 50,"added my tag to to header in request" );
+			$param->{tag} = $self->{local_tag};
+			$to = sip_parts2hdrval( 'to',$data,$param );
+			$request->set_header( to => $to );
+		}
+	}
+
 	if ( $method eq 'BYE' || $method eq 'CANCEL' ) {
 		# if the peer wants to hangup we must confirm
 		my $response = $request->create_response( '200','Closing' );
