@@ -42,10 +42,13 @@ use fields qw( call_cleanup rtp_cleanup ctx param );
 #   cb_established: callback which will be called on receiving ACK in INVITE
 #       with (status,self) where status is OK|FAIL
 #   sip_header: hashref of SIP headers to add
+#   call_on_hold: one-shot parameter to set local media addr to 0.0.0.0,
+#       will be set to false after use
 
 use Net::SIP::Util qw(create_rtp_sockets invoke_callback);
 use Net::SIP::Debug;
 use Socket;
+use Storable 'dclone';
 
 ###########################################################################
 # create a new call based on a controller
@@ -143,7 +146,7 @@ sub reinvite {
 	$clear_sdp = $param->{clear_sdp} if ! defined $clear_sdp;
 	if ( $clear_sdp ) {
 		# clear SDP keys so that a new SDP session will be created
-		@{ $param }{qw( sdp sdp_peer media_ssocks media_lsocks )} = ()
+		@{ $param }{qw( sdp _sdp_saved sdp_peer media_ssocks media_lsocks )} = ()
 	}
 	$self->{param} = $param = { %$param, %args } if %args;
 
@@ -167,15 +170,15 @@ sub reinvite {
 		my Net::SIP::Simple::Call $self = shift;
 		my ($endpoint,$ctx,$errno,$code,$packet,$leg,$from,$ack) = @_;
 
-		# new requests in existing call are handled in receive()
-		return $self->receive( @_ ) if $packet->is_request;
-
 		if ( $errno ) {
-			$self->error( "Failed with error $errno code=$code" );
+			$self->error( "Failed with error $errno".( $code ? " code=$code" :"" ) );
 			invoke_callback( $param->{cb_final}, 'FAIL',$self,errno => $errno,code => $code,packet => $packet );
 			return;
 		}
 
+		# new requests in existing call are handled in receive()
+		return $self->receive( @_ ) if $packet->is_request;
+
 		# response to INVITE
 		# all other responses will not be propagated to this callback
 		my $param = $self->{param};
@@ -297,7 +300,7 @@ sub receive {
 
 				if ( $param->{clear_sdp} ) {
 					# clear SDP keys so that a new SDP session will be created
-					@{ $param }{qw( sdp sdp_peer media_ssocks media_lsocks )} = ()
+					@{ $param }{qw( sdp _sdp_saved sdp_peer media_ssocks media_lsocks )} = ()
 				}
 
 				$param->{leg} ||= $leg;
@@ -356,12 +359,18 @@ sub _setup_peer_rtp_socks {
 	}
 
 	my $raddr = $param->{media_raddr} = [];
+	my $null_address = pack( 'CCCC',0,0,0,0 ); # c=0.0.0.0 => call on hold
 	foreach my $m (@media) {
 		my $range = $m->{range} || 1;
-		my @socks = map {
-			scalar(sockaddr_in( $m->{port} + $_ ,inet_aton( $m->{addr} )));
-		} (0..$range-1);
-		push @$raddr, @socks == 1 ? $socks[0] : \@socks;
+		my $paddr = inet_aton( $m->{addr} );
+		if ( $paddr eq $null_address ) {
+			# on-hold for this media
+			push @$raddr, undef;
+		} else {
+			my @socks = map { scalar(sockaddr_in( $m->{port}+$_ , $paddr )) }
+				(0..$range-1);
+			push @$raddr, @socks == 1 ? $socks[0] : \@socks;
+		}
 	}
 
 	return 1;
@@ -377,7 +386,10 @@ sub _setup_local_rtp_socks {
 	my Net::SIP::Simple::Call $self = shift;
 	my $param = $self->{param};
 
-	my $sdp = $param->{sdp};
+	my $call_on_hold = $param->{call_on_hold};
+	$param->{call_on_hold} = 0; # one-shot
+
+	my $sdp = $param->{_sdp_saved} || $param->{sdp};
 	if ( $sdp && !UNIVERSAL::isa( $sdp,'Net::SIP::SDP' )) {
 		$sdp = Net::SIP::SDP->new( $sdp );
 	}
@@ -450,6 +462,14 @@ sub _setup_local_rtp_socks {
 			push @$msocks,$socks;
 		}
 	}
+
+	$param->{_sdp_saved} = $sdp;
+	if ( $call_on_hold ) {
+		$sdp = dclone($sdp); # make changes on clone
+		my @new = map { [ '0.0.0.0',$_->{port} ] } $sdp->get_media;
+		$sdp->replace_media_listen( @new );
+		$param->{sdp} = $sdp;
+	}
 }
 
 1;
